@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith8.cc,v 1.36 2005/05/20 20:06:50 sshwarts Exp $
+// $Id: arith8.cc,v 1.37 2005/07/21 01:59:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -100,8 +100,7 @@ void BX_CPU_C::ADC_EbGb(bxInstruction_c *i)
     Write_RMW_virtual_byte(sum);
   }
 
-  SET_FLAGS_OSZAPC_8(op1, op2, sum,
-	       (temp_CF) ? BX_INSTR_ADC8 : BX_INSTR_ADD8);
+  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
 }
 
 void BX_CPU_C::ADC_GbEb(bxInstruction_c *i)
@@ -120,8 +119,7 @@ void BX_CPU_C::ADC_GbEb(bxInstruction_c *i)
 
   sum = op1 + op2 + temp_CF;
 
-  SET_FLAGS_OSZAPC_8(op1, op2, sum,
-	       (temp_CF) ? BX_INSTR_ADC8 : BX_INSTR_ADD8);
+  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
 
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), sum);
 }
@@ -136,8 +134,7 @@ void BX_CPU_C::ADC_ALIb(bxInstruction_c *i)
   sum = op1 + op2 + temp_CF;
   AL = sum;
 
-  SET_FLAGS_OSZAPC_8(op1, op2, sum,
-	       (temp_CF) ? BX_INSTR_ADC8 : BX_INSTR_ADD8);
+  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
 }
 
 void BX_CPU_C::SBB_EbGb(bxInstruction_c *i)
@@ -158,8 +155,7 @@ void BX_CPU_C::SBB_EbGb(bxInstruction_c *i)
     Write_RMW_virtual_byte(diff_8);
   }
 
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8,
-	       (temp_CF) ? BX_INSTR_SBB8 : BX_INSTR_SUB8);
+  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
 }
 
 void BX_CPU_C::SBB_GbEb(bxInstruction_c *i)
@@ -180,8 +176,7 @@ void BX_CPU_C::SBB_GbEb(bxInstruction_c *i)
 
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), diff_8);
 
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8,
-	       (temp_CF) ? BX_INSTR_SBB8 : BX_INSTR_SUB8);
+  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
 }
 
 void BX_CPU_C::SBB_ALIb(bxInstruction_c *i)
@@ -194,8 +189,7 @@ void BX_CPU_C::SBB_ALIb(bxInstruction_c *i)
   diff_8 = op1_8 - (op2_8 + temp_CF);
   AL = diff_8;
 
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8,
-	       (temp_CF) ? BX_INSTR_SBB8 : BX_INSTR_SUB8);
+  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
 }
 
 void BX_CPU_C::SBB_EbIb(bxInstruction_c *i)
@@ -216,8 +210,7 @@ void BX_CPU_C::SBB_EbIb(bxInstruction_c *i)
     Write_RMW_virtual_byte(diff_8);
   }
 
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8,
-	       (temp_CF) ? BX_INSTR_SBB8 : BX_INSTR_SUB8);
+  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
 }
 
 void BX_CPU_C::SUB_EbGb(bxInstruction_c *i)
@@ -409,8 +402,7 @@ void BX_CPU_C::ADC_EbIb(bxInstruction_c *i)
     Write_RMW_virtual_byte(sum);
   }
 
-  SET_FLAGS_OSZAPC_8(op1, op2, sum,
-	       (temp_CF) ? BX_INSTR_ADC8 : BX_INSTR_ADD8);
+  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
 }
 
 void BX_CPU_C::SUB_EbIb(bxInstruction_c *i)
