@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ctrl_xfer8.cc,v 1.17 2004/11/02 18:05:19 sshwarts Exp $
+// $Id: ctrl_xfer8.cc,v 1.19 2005/05/20 20:06:50 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -31,7 +31,6 @@
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
-
 #if BX_SUPPORT_X86_64==0
 // Make life a little easier for the 64/32-bit merge.
 #define RCX ECX
@@ -39,26 +38,24 @@
 #endif
 
 
-  void
-BX_CPU_C::JCXZ_Jb(bxInstruction_c *i)
+void BX_CPU_C::JCXZ_Jb(bxInstruction_c *i)
 {
-BailBigRSP("JCXZ_Jb");
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
     if ( RCX == 0 ) {
       RIP += (Bit32s) i->Id();
       BX_INSTR_CNEAR_BRANCH_TAKEN(BX_CPU_ID, RIP);
       revalidate_prefetch_q();
-      }
+    }
 #if BX_INSTRUMENTATION
     else {
       BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
-      }
-#endif
     }
+#endif
+  }
   else
 #endif
-   {
+  {
     Bit32u temp_ECX;
 
     if (i->as32L())
@@ -71,36 +68,33 @@ BailBigRSP("JCXZ_Jb");
       if (i->os32L()==0) new_EIP &= 0x0000ffff;
       branch_near32(new_EIP);
       BX_INSTR_CNEAR_BRANCH_TAKEN(BX_CPU_ID, new_EIP);
-      }
+    }
 #if BX_INSTRUMENTATION
     else {
       BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
-      }
+    }
 #endif
   }
 }
 
-  void
-BX_CPU_C::LOOPNE_Jb(bxInstruction_c *i)
+void BX_CPU_C::LOOPNE_Jb(bxInstruction_c *i)
 {
-BailBigRSP("loopne_jb");
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
-
     if ( ((--RCX)!=0) && (get_ZF()==0) ) {
       RIP += (Bit32s) i->Id();
       BX_INSTR_CNEAR_BRANCH_TAKEN(BX_CPU_ID, RIP);
       revalidate_prefetch_q();
-      }
+    }
 #if BX_INSTRUMENTATION
     else {
       BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
-      }
-#endif
     }
+#endif
+  }
   else
 #endif
-   {
+  {
     Bit32u count;
 
 #if BX_CPU_LEVEL >= 3
@@ -120,7 +114,7 @@ BailBigRSP("loopne_jb");
 #if BX_INSTRUMENTATION
     else {
       BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
-      }
+    }
 #endif
 
     if (i->as32L())
@@ -130,10 +124,8 @@ BailBigRSP("loopne_jb");
     }
 }
 
-  void
-BX_CPU_C::LOOPE_Jb(bxInstruction_c *i)
+void BX_CPU_C::LOOPE_Jb(bxInstruction_c *i)
 {
-BailBigRSP("loope_jb");
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
     if ( ((--RCX)!=0) && (get_ZF()) ) {
@@ -144,9 +136,9 @@ BailBigRSP("loope_jb");
 #if BX_INSTRUMENTATION
     else {
       BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
-      }
-#endif
     }
+#endif
+  }
   else
 #endif
  {
@@ -169,36 +161,34 @@ BailBigRSP("loope_jb");
 #if BX_INSTRUMENTATION
     else {
       BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
-      }
+    }
 #endif
 
     if (i->as32L())
       RCX = ECX - 1;   // zero extend
     else
       CX--;
-    }
+  }
 }
 
-  void
-BX_CPU_C::LOOP_Jb(bxInstruction_c *i)
+void BX_CPU_C::LOOP_Jb(bxInstruction_c *i)
 {
-BailBigRSP("loop_jb");
 #if BX_SUPPORT_X86_64
   if (i->as64L()) {
     if ( ((--RCX)!=0) ) {
       RIP += (Bit32s) i->Id();
       BX_INSTR_CNEAR_BRANCH_TAKEN(BX_CPU_ID, RIP);
       revalidate_prefetch_q();
-      }
+    }
 #if BX_INSTRUMENTATION
     else {
       BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
-      }
-#endif
     }
+#endif
+  }
   else
 #endif
- {
+  {
     Bit32u count;
 
 #if BX_CPU_LEVEL >= 3
@@ -218,12 +208,12 @@ BailBigRSP("loop_jb");
 #if BX_INSTRUMENTATION
     else {
       BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
-      }
+    }
 #endif
 
     if (i->as32L())
       RCX = ECX - 1;         // zero extend
     else
       CX--;
-    }
+  }
 }
