@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical32.cc,v 1.19 2002/10/25 18:26:28 sshwarts Exp $
+// $Id: logical32.cc,v 1.24 2005/05/20 20:06:50 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,19 +25,12 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
-
-
-
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
-
-
-  void
-BX_CPU_C::XOR_EdGd(bxInstruction_c *i)
+void BX_CPU_C::XOR_EdGd(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32, result_32;
 
@@ -47,19 +40,17 @@ BX_CPU_C::XOR_EdGd(bxInstruction_c *i)
     op1_32 = BX_READ_32BIT_REG(i->rm());
     result_32 = op1_32 ^ op2_32;
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
-    }
+  }
   else {
     read_RMW_virtual_dword(i->seg(), RMAddr(i), &op1_32);
     result_32 = op1_32 ^ op2_32;
     Write_RMW_virtual_dword(result_32);
-    }
+  }
 
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_XOR32);
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 }
 
-
-  void
-BX_CPU_C::XOR_GdEd(bxInstruction_c *i)
+void BX_CPU_C::XOR_GdEd(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, result_32;
   unsigned nnn = i->nnn();
@@ -68,21 +59,19 @@ BX_CPU_C::XOR_GdEd(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_32 = BX_READ_32BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_dword(i->seg(), RMAddr(i), &op2_32);
-    }
+  }
 
   result_32 = op1_32 ^ op2_32;
 
   BX_WRITE_32BIT_REGZ(nnn, result_32);
 
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_XOR32);
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 }
 
-
-  void
-BX_CPU_C::XOR_EAXId(bxInstruction_c *i)
+void BX_CPU_C::XOR_EAXId(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, sum_32;
 
@@ -97,11 +86,10 @@ BX_CPU_C::XOR_EAXId(bxInstruction_c *i)
   EAX = sum_32;
 #endif
 
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, sum_32, BX_INSTR_XOR32);
+  SET_FLAGS_OSZAPC_RESULT_32(sum_32, BX_INSTR_LOGIC32);
 }
 
-  void
-BX_CPU_C::XOR_EdId(bxInstruction_c *i)
+void BX_CPU_C::XOR_EdId(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32, result_32;
 
@@ -111,19 +99,17 @@ BX_CPU_C::XOR_EdId(bxInstruction_c *i)
     op1_32 = BX_READ_32BIT_REG(i->rm());
     result_32 = op1_32 ^ op2_32;
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
-    }
+  }
   else {
     read_RMW_virtual_dword(i->seg(), RMAddr(i), &op1_32);
     result_32 = op1_32 ^ op2_32;
     Write_RMW_virtual_dword(result_32);
-    }
+  }
 
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_XOR32);
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 }
 
-
-  void
-BX_CPU_C::OR_EdId(bxInstruction_c *i)
+void BX_CPU_C::OR_EdId(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32, result_32;
 
@@ -133,18 +119,17 @@ BX_CPU_C::OR_EdId(bxInstruction_c *i)
     op1_32 = BX_READ_32BIT_REG(i->rm());
     result_32 = op1_32 | op2_32;
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
-    }
+  }
   else {
     read_RMW_virtual_dword(i->seg(), RMAddr(i), &op1_32);
     result_32 = op1_32 | op2_32;
     Write_RMW_virtual_dword(result_32);
-    }
+  }
 
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_OR32);
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 }
 
-  void
-BX_CPU_C::NOT_Ed(bxInstruction_c *i)
+void BX_CPU_C::NOT_Ed(bxInstruction_c *i)
 {
   Bit32u op1_32, result_32;
 
@@ -152,17 +137,15 @@ BX_CPU_C::NOT_Ed(bxInstruction_c *i)
     op1_32 = BX_READ_32BIT_REG(i->rm());
     result_32 = ~op1_32;
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
-    }
+  }
   else {
     read_RMW_virtual_dword(i->seg(), RMAddr(i), &op1_32);
     result_32 = ~op1_32;
     Write_RMW_virtual_dword(result_32);
-    }
+  }
 }
 
-
-  void
-BX_CPU_C::OR_EdGd(bxInstruction_c *i)
+void BX_CPU_C::OR_EdGd(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32, result_32;
 
@@ -172,19 +155,17 @@ BX_CPU_C::OR_EdGd(bxInstruction_c *i)
     op1_32 = BX_READ_32BIT_REG(i->rm());
     result_32 = op1_32 | op2_32;
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
-    }
+  }
   else {
     read_RMW_virtual_dword(i->seg(), RMAddr(i), &op1_32);
     result_32 = op1_32 | op2_32;
     Write_RMW_virtual_dword(result_32);
-    }
+  }
 
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_OR32);
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 }
 
-
-  void
-BX_CPU_C::OR_GdEd(bxInstruction_c *i)
+void BX_CPU_C::OR_GdEd(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, result_32;
 
@@ -192,36 +173,29 @@ BX_CPU_C::OR_GdEd(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_32 = BX_READ_32BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_dword(i->seg(), RMAddr(i), &op2_32);
-    }
+  }
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Or32)
   Bit32u flags32;
-
   asmOr32(result_32, op1_32, op2_32, flags32);
   setEFlagsOSZAPC(flags32);
 #else
   result_32 = op1_32 | op2_32;
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 #endif
 
   BX_WRITE_32BIT_REGZ(i->nnn(), result_32);
-
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_OR32);
-#endif
 }
 
-
-  void
-BX_CPU_C::OR_EAXId(bxInstruction_c *i)
+void BX_CPU_C::OR_EAXId(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, sum_32;
 
   op1_32 = EAX;
   op2_32 = i->Id();
-
   sum_32 = op1_32 | op2_32;
 
 #if BX_SUPPORT_X86_64
@@ -230,13 +204,10 @@ BX_CPU_C::OR_EAXId(bxInstruction_c *i)
   EAX = sum_32;
 #endif
 
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, sum_32, BX_INSTR_OR32);
+  SET_FLAGS_OSZAPC_RESULT_32(sum_32, BX_INSTR_LOGIC32);
 }
 
-
-
-  void
-BX_CPU_C::AND_EdGd(bxInstruction_c *i)
+void BX_CPU_C::AND_EdGd(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32, result_32;
 
@@ -245,9 +216,8 @@ BX_CPU_C::AND_EdGd(bxInstruction_c *i)
   if (i->modC0()) {
     op1_32 = BX_READ_32BIT_REG(i->rm());
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And32)
     Bit32u flags32;
-
     asmAnd32(result_32, op1_32, op2_32, flags32);
     setEFlagsOSZAPC(flags32);
 #else
@@ -255,13 +225,12 @@ BX_CPU_C::AND_EdGd(bxInstruction_c *i)
 #endif
 
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
-    }
+  }
   else {
     read_RMW_virtual_dword(i->seg(), RMAddr(i), &op1_32);
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And32)
     Bit32u flags32;
-
     asmAnd32(result_32, op1_32, op2_32, flags32);
     setEFlagsOSZAPC(flags32);
 #else
@@ -269,16 +238,14 @@ BX_CPU_C::AND_EdGd(bxInstruction_c *i)
 #endif
 
     Write_RMW_virtual_dword(result_32);
-    }
+  }
 
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_AND32);
+#if !defined(BX_HostAsm_And32)
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 #endif
 }
 
-
-  void
-BX_CPU_C::AND_GdEd(bxInstruction_c *i)
+void BX_CPU_C::AND_GdEd(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, result_32;
 
@@ -286,43 +253,37 @@ BX_CPU_C::AND_GdEd(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_32 = BX_READ_32BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_dword(i->seg(), RMAddr(i), &op2_32);
-    }
+  }
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And32)
   Bit32u flags32;
-
   asmAnd32(result_32, op1_32, op2_32, flags32);
   setEFlagsOSZAPC(flags32);
 #else
   result_32 = op1_32 & op2_32;
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 #endif
 
   BX_WRITE_32BIT_REGZ(i->nnn(), result_32);
-
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_AND32);
-#endif
 }
 
-
-  void
-BX_CPU_C::AND_EAXId(bxInstruction_c *i)
+void BX_CPU_C::AND_EAXId(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, result_32;
 
   op1_32 = EAX;
   op2_32 = i->Id();
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And32)
   Bit32u flags32;
-
   asmAnd32(result_32, op1_32, op2_32, flags32);
   setEFlagsOSZAPC(flags32);
 #else
   result_32 = op1_32 & op2_32;
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 #endif
 
 #if BX_SUPPORT_X86_64
@@ -330,14 +291,9 @@ BX_CPU_C::AND_EAXId(bxInstruction_c *i)
 #else
   EAX = result_32;
 #endif
-
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_AND32);
-#endif
 }
 
-  void
-BX_CPU_C::AND_EdId(bxInstruction_c *i)
+void BX_CPU_C::AND_EdId(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32, result_32;
 
@@ -346,9 +302,8 @@ BX_CPU_C::AND_EdId(bxInstruction_c *i)
   if (i->modC0()) {
     op1_32 = BX_READ_32BIT_REG(i->rm());
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And32)
     Bit32u flags32;
-
     asmAnd32(result_32, op1_32, op2_32, flags32);
     setEFlagsOSZAPC(flags32);
 #else
@@ -356,13 +311,12 @@ BX_CPU_C::AND_EdId(bxInstruction_c *i)
 #endif
 
     BX_WRITE_32BIT_REGZ(i->rm(), result_32);
-    }
+  }
   else {
     read_RMW_virtual_dword(i->seg(), RMAddr(i), &op1_32);
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And32)
     Bit32u flags32;
-
     asmAnd32(result_32, op1_32, op2_32, flags32);
     setEFlagsOSZAPC(flags32);
 #else
@@ -370,16 +324,14 @@ BX_CPU_C::AND_EdId(bxInstruction_c *i)
 #endif
 
     Write_RMW_virtual_dword(result_32);
-    }
+  }
 
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_AND32);
+#if !defined(BX_HostAsm_And32)
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 #endif
 }
 
-
-  void
-BX_CPU_C::TEST_EdGd(bxInstruction_c *i)
+void BX_CPU_C::TEST_EdGd(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32;
 
@@ -387,50 +339,39 @@ BX_CPU_C::TEST_EdGd(bxInstruction_c *i)
 
   if (i->modC0()) {
     op1_32 = BX_READ_32BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_dword(i->seg(), RMAddr(i), &op1_32);
-    }
+  }
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Test32)
   Bit32u flags32;
-
   asmTest32(op1_32, op2_32, flags32);
   setEFlagsOSZAPC(flags32);
 #else
-  Bit32u result_32;
-  result_32 = op1_32 & op2_32;
-
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_TEST32);
+  Bit32u result_32 = op1_32 & op2_32;
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 #endif
 }
 
-
-
-  void
-BX_CPU_C::TEST_EAXId(bxInstruction_c *i)
+void BX_CPU_C::TEST_EAXId(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32;
 
   op1_32 = EAX;
   op2_32 = i->Id();
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Test32)
   Bit32u flags32;
-
   asmTest32(op1_32, op2_32, flags32);
   setEFlagsOSZAPC(flags32);
 #else
-  Bit32u result_32;
-  result_32 = op1_32 & op2_32;
-
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_TEST32);
+  Bit32u result_32 = op1_32 & op2_32;
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 #endif
 }
 
-
-  void
-BX_CPU_C::TEST_EdId(bxInstruction_c *i)
+void BX_CPU_C::TEST_EdId(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32;
 
@@ -438,20 +379,17 @@ BX_CPU_C::TEST_EdId(bxInstruction_c *i)
 
   if (i->modC0()) {
     op1_32 = BX_READ_32BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_dword(i->seg(), RMAddr(i), &op1_32);
-    }
+  }
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Test32)
   Bit32u flags32;
-
   asmTest32(op1_32, op2_32, flags32);
   setEFlagsOSZAPC(flags32);
 #else
-  Bit32u result_32;
-  result_32 = op1_32 & op2_32;
-
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, result_32, BX_INSTR_TEST32);
+  Bit32u result_32 = op1_32 & op2_32;
+  SET_FLAGS_OSZAPC_RESULT_32(result_32, BX_INSTR_LOGIC32);
 #endif
 }
