@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: protect_ctrl_pro.cc,v 1.7 2004/08/13 20:00:03 sshwarts Exp $
+// $Id: protect_ctrl_pro.cc,v 1.8 2005/01/28 20:50:48 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -30,8 +30,7 @@
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
-  void
-BX_CPU_C::enter_protected_mode(void)
+void BX_CPU_C::enter_protected_mode(void)
 {
 // BX_INFO(("processor switching into PROTECTED mode!!!"));
 // debug(BX_CPU_THIS_PTR prev_eip);
@@ -40,13 +39,12 @@ BX_CPU_C::enter_protected_mode(void)
   if (bx_dbg.reset)
     BX_INFO(("processor switching into PROTECTED mode!!!"));
 
-if ( BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.rpl!=0 || BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.rpl!=0 )
-  BX_PANIC(("enter_protected_mode: CS or SS rpl != 0"));
+  if (BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.rpl!=0 || BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.rpl!=0)
+    BX_PANIC(("enter_protected_mode: CS or SS rpl != 0"));
 }
 
 
-  void
-BX_CPU_C::enter_real_mode(void)
+void BX_CPU_C::enter_real_mode(void)
 {
 // BX_INFO(("processor switching into REAL mode!!!"));
 // debug(BX_CPU_THIS_PTR prev_eip);
@@ -55,6 +53,6 @@ BX_CPU_C::enter_real_mode(void)
   if (bx_dbg.reset)
     BX_INFO(("processor switching into REAL mode!!!"));
 
-if ( BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.rpl!=0 || BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.rpl!=0 )
-  BX_PANIC(("enter_real_mode: CS or SS rpl != 0"));
+  if (BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.rpl!=0 || BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.rpl!=0)
+    BX_PANIC(("enter_real_mode: CS or SS rpl != 0"));
 }
