@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sse_move.cc,v 1.46 2005/09/24 16:56:20 sshwarts Exp $
+// $Id: sse_move.cc,v 1.47 2006/01/24 19:03:54 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -43,6 +43,16 @@ void BX_CPU_C::prepareSSE(void)
 
 #define BX_MXCSR_REGISTER (BX_CPU_THIS_PTR mxcsr.mxcsr)
 
+void BX_CPU_C::print_state_SSE(void)
+{
+  fprintf(stderr, "MXCSR: %08x\n", BX_MXCSR_REGISTER);
+  for(unsigned i=0;i<BX_XMM_REGISTERS;i++) {
+    BxPackedXmmRegister xmm = BX_READ_XMM_REG(i);
+    fprintf(stderr, "XMM%02u: %08x%08x:%08x%08x\n", i,
+       xmm.xmm32u(3), xmm.xmm32u(2), xmm.xmm32u(1), xmm.xmm32u(0));
+  }
+}
+
 #endif
 
 /* ************************************ */
