@@ -66,7 +66,7 @@ enum {
 };
 
 class disassembler;
-class x86_insn;
+struct x86_insn;
 
 typedef void (disassembler::*BxDisasmPtr_t)(const x86_insn *insn);
 typedef void (disassembler::*BxDisasmResolveModrmPtr_t)(const x86_insn *insn, unsigned attr);
@@ -164,6 +164,8 @@ public:
   void set_syntax_intel();
   void set_syntax_att  ();
 
+  void toggle_syntax_mode();
+
 private:
   bx_bool intel_mode;
 
