@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////
-// $Id: wxdialog.cc,v 1.76 2005/05/17 18:05:49 vruppert Exp $
+// $Id: wxdialog.cc,v 1.80 2005/11/26 09:22:58 vruppert Exp $
 /////////////////////////////////////////////////////////////////
 
 // Define BX_PLUGGABLE in files that can be compiled into plugins.  For
@@ -184,6 +184,7 @@ void LogMsgAskDialog::ShowHelp ()
 BEGIN_EVENT_TABLE(FloppyConfigDialog, wxDialog)
   EVT_BUTTON(-1, FloppyConfigDialog::OnEvent)
   EVT_TEXT(-1, FloppyConfigDialog::OnEvent)
+  EVT_CHOICE(-1, FloppyConfigDialog::OnEvent)
 END_EVENT_TABLE()
 
 
@@ -211,7 +212,7 @@ FloppyConfigDialog::FloppyConfigDialog(
   vertSizer->Add (buttonSizer, 0, wxALIGN_RIGHT|wxTOP, 30);
   // contents of capacitySizer
   wxStaticText *captext = new wxStaticText (this, -1, FLOPPY_CONFIG_CAP);
-  capacity = new wxChoice (this, -1);
+  capacity = new wxChoice (this, ID_Capacity);
   capacitySizer->Add (captext, 0, wxALL, 5);
   capacitySizer->Add (capacity, 0, wxALL|wxADJUST_MINSIZE, 5);
   // contents of buttonSizer
@@ -220,8 +221,8 @@ FloppyConfigDialog::FloppyConfigDialog(
   // use wxID_CANCEL because pressing ESC produces this same code
   btn = new wxButton (this, wxID_CANCEL, "Cancel");
   buttonSizer->Add (btn, 0, wxALL, 5);
-  btn = new wxButton (this, ID_Create, "Create Image");
-  buttonSizer->Add (btn, 0, wxALL, 5);
+  CreateBtn = new wxButton (this, ID_Create, "Create Image");
+  buttonSizer->Add (CreateBtn, 0, wxALL, 5);
   btn = new wxButton (this, wxID_OK, "Ok");
   buttonSizer->Add (btn, 0, wxALL, 5);
   // create filename and diskImageRadioBtn so that we can tweak them before
@@ -262,6 +263,12 @@ void FloppyConfigDialog::SetCapacityChoices (int n, char *choices[])
     capacity->Append (wxString (choices[i]));
 }
 
+void FloppyConfigDialog::SetCapacity (int cap)
+{
+  capacity->SetSelection(cap);
+  CreateBtn->Enable(floppy_type_n_sectors[cap] > 0);
+}
+
 void FloppyConfigDialog::Init()
 {
   // add contents of diskImageSizer
@@ -331,7 +338,6 @@ FloppyConfigDialog::GetFilename ()
 void FloppyConfigDialog::OnEvent(wxCommandEvent& event)
 {
   int id = event.GetId ();
-  wxLogMessage ("you pressed button id=%d", id);
   switch (id) {
     case ID_FilenameText:
       // when you type into the filename field, ensure that the radio
@@ -345,25 +351,28 @@ void FloppyConfigDialog::OnEvent(wxCommandEvent& event)
       EndModal (wxID_OK);
       break;
     case ID_Browse:
-      BrowseTextCtrl (filename);
+      if (BrowseTextCtrl(filename)) {
+        capacity->SetSelection(capacity->FindString("auto"));
+      }
+      break;
+    case ID_Capacity:
+      {
+        int cap = capacity->GetSelection();
+        CreateBtn->Enable(floppy_type_n_sectors[cap] > 0);
+      }
       break;
     case ID_Create:
       {
-	int cap = capacity->GetSelection ();
-	if (capacity->GetString (cap).Cmp ("none") == 0
-	    || !(cap>=0 && cap<n_floppy_type_names)) {
-	  wxMessageBox("You must choose a valid capacity for the new disk image", "Bad Capacity", wxOK | wxICON_ERROR, this );
-	  return;
-	}
-	char name[1024];
-	strncpy (name, filename->GetValue ().c_str (), sizeof(name));
+        int cap = capacity->GetSelection ();
+        char name[1024];
+        strncpy (name, filename->GetValue ().c_str (), sizeof(name));
         if (CreateImage (0, floppy_type_n_sectors[cap], name)) {
-	  wxString msg;
-	  msg.Printf ("Created a %s disk image called '%s'.",
-	      capacity->GetString (cap).c_str (), 
-	      filename->GetValue ().c_str ());
-	  wxMessageBox(msg, "Image Created", wxOK | wxICON_INFORMATION, this);
-	}
+          wxString msg;
+          msg.Printf ("Created a %s disk image called '%s'.",
+            capacity->GetString (cap).c_str (), 
+            filename->GetValue ().c_str ());
+          wxMessageBox(msg, "Image Created", wxOK | wxICON_INFORMATION, this);
+        }
       }
       break;
     case wxID_CANCEL:
@@ -1161,6 +1170,7 @@ void ParamDialog::AddParam (
           txtctrl->SetValue (buffer);
         } else {
           txtctrl->SetValue (param->getptr ());
+          txtctrl->SetMaxLength (param->get_maxsize ());
         }
 	sizer->Add (txtctrl, 0, wxALL, 2);
 	if (!plain) {
@@ -2010,11 +2020,12 @@ int GetTextCtrlInt (wxTextCtrl *ctrl,
 bool BrowseTextCtrl (wxTextCtrl *text, wxString prompt, long style) {
   // try to configure the dialog to show hidden files
   wxConfigBase::Get() -> Write(wxT("/wxWidgets/wxFileDialog/ShowHidden"), true);
-  wxFileDialog *fdialog = new wxFileDialog (text->GetParent (), prompt, "", text->GetValue (), wxString(), style);
-  if (fdialog->ShowModal () == wxID_OK)
+  wxFileDialog *fdialog = new wxFileDialog (text->GetParent (), prompt, "", text->GetValue (), "*.*", style);
+  int result = fdialog->ShowModal();
+  if (result == wxID_OK)
     text->SetValue (fdialog->GetPath ());
   delete fdialog;
-  return true;
+  return (result == wxID_OK);
 }
 
 wxChoice *makeLogOptionChoiceBox (wxWindow *parent,
