@@ -1,8 +1,8 @@
 ////////////////////////////////////////////////////////////////////
-// $Id: wxdialog.h,v 1.53 2003/09/09 16:41:25 vruppert Exp $
+// $Id: wxdialog.h,v 1.56 2005/11/25 16:24:47 vruppert Exp $
 ////////////////////////////////////////////////////////////////////
 //
-// wxWindows dialogs for Bochs
+// wxWidgets dialogs for Bochs
 
 #include <wx/spinctrl.h>
 
@@ -171,7 +171,8 @@ public:
 #define FLOPPY_CONFIG_TITLE "Configure %s"
 #define FLOPPY_CONFIG_INSTRS "Select the device or image to use when simulating %s."
 #define FLOPPY_CONFIG_CAP "What is the capacity of this disk?"
-#define FLOPPY_CONFIG_HINT "To create a disk image, choose the file name and capacity, then click on \"Create Image\"."
+#define FLOPPY_CONFIG_HINT "To create a disk image, choose the file name and capacity, then click on \"Create Image\".\n\n" \
+                           "Clicking OK signals a media change for this drive."
 #define FLOPPY_CONFIG_DISKIMG "Disk image: "
 private:
   void Init ();  // called automatically by ShowModal()
@@ -181,6 +182,7 @@ private:
   wxRadioButton *rbtn[FLOPPY_MAX_RBTNS];
   wxString equivalentFilename[FLOPPY_MAX_RBTNS];
   int n_rbtns;
+  wxButton *CreateBtn;
   wxRadioButton *diskImageRadioBtn;
   wxTextCtrl *filename;
   wxChoice *capacity;
@@ -197,7 +199,7 @@ public:
   // Use char* instead of wxString because the array we use is already
   // expressed as a char *[].
   void SetCapacityChoices (int n, char *choices[]);
-  void SetCapacity (int cap) { capacity->SetSelection (cap); }
+  void SetCapacity (int cap);
   int GetRadio ();
   int GetCapacity () { return capacity->GetSelection (); }
   wxString GetFilename ();
@@ -467,7 +469,7 @@ private:
   bool runtime;
 protected:
   wxBoxSizer *mainSizer, *buttonSizer;
-  // hash table that maps the ID of a wxWindows control (e.g. wxChoice,
+  // hash table that maps the ID of a wxWidgets control (e.g. wxChoice,
   // wxTextCtrl) to the associated ParamStruct object.  Data in the hash table
   // is of ParamStruct*.
   wxHashTable *idHash;
@@ -523,7 +525,7 @@ DECLARE_EVENT_TABLE()
 // enable different groups like debug, FPU, MMX registers.  Certainly if you
 // interrupt the simulation, these would be updated.  we could update
 // periodically during simulation if it was useful.  If we get the debugger
-// integrated with wxwindows, you could single step and update the cpu
+// integrated with wxwidgets, you could single step and update the cpu
 // registers, with regs that change marked in a different color.  Modeless
 // dialog.
 // 
@@ -624,7 +626,7 @@ Everything else in here is a comment!
 
 Here are some quick sketches of what different parts of the interface
 could look like.  None of these is implemented yet, and everything is
-open for debate.  Whoever writes the wxwindows code for any of these
+open for debate.  Whoever writes the wxwidgets code for any of these
 screens gets several thousand votes!
 
 
