@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cmos.h,v 1.9 2003/01/04 00:02:07 vruppert Exp $
+// $Id: cmos.h,v 1.13 2005/12/04 17:43:09 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -25,9 +25,6 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
-
-
-
 #if BX_USE_CMOS_SMF
 #  define BX_CMOS_SMF  static
 #  define BX_CMOS_THIS theCmosDevice->
@@ -45,6 +42,7 @@ public:
   virtual void init(void);
   virtual void checksum_cmos(void);
   virtual void reset(unsigned type);
+  virtual void save_image(void);
 
   virtual Bit32u get_reg(unsigned reg) {
     return s.reg[reg];
@@ -64,9 +62,11 @@ public:
     time_t  timeval;
     Bit8u   cmos_mem_address;
     bx_bool timeval_change;
+    bx_bool rtc_mode_12hour;
+    bx_bool rtc_mode_binary;
 
-    Bit8u   reg[BX_NUM_CMOS_REGS];
-    } s;  // state information
+    Bit8u   reg[128];
+  } s;  // state information
 
 private:
   static Bit32u read_handler(void *this_ptr, Bit32u address, unsigned io_len);
