@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ioapic.h,v 1.11 2005/12/26 19:42:09 sshwarts Exp $
+// $Id: ioapic.h,v 1.15 2006/01/10 06:13:26 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 
 extern class bx_ioapic_c bx_ioapic;
@@ -36,6 +36,7 @@ public:
 
 class bx_ioapic_c : public bx_generic_apic_c {
   Bit32u ioregsel;    // selects between various registers
+  Bit32u intin;
   // interrupt request bitmask, not visible from the outside.  Bits in the
   // irr are set when trigger_irq is called, and cleared when the interrupt
   // is delivered to the processor.  If an interrupt is masked, the irr
@@ -51,8 +52,8 @@ public:
   virtual void reset (unsigned type);
   virtual void read_aligned(Bit32u address, Bit32u *data, unsigned len);
   virtual void write(Bit32u address, Bit32u *value, unsigned len);
-  void raise_irq (unsigned num, unsigned from);
-  void lower_irq (unsigned num, unsigned from);
+  void set_irq_level(Bit8u int_in, bx_bool level);
+  void receive_eoi(Bit8u vector);
   void service_ioapic ();
   virtual bx_apic_type_t get_type () { return APIC_TYPE_IOAPIC; }
 };
