@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: svga_cirrus.h,v 1.5 2005/04/13 18:39:26 vruppert Exp $
+// $Id: svga_cirrus.h,v 1.6 2005/11/27 17:49:59 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 // Copyright (c) 2004 Makoto Suzuki (suzu)
@@ -74,7 +74,6 @@ public:
   virtual void get_text_snapshot(Bit8u **text_snapshot, 
                                  unsigned *txHeight, unsigned *txWidth);
   virtual void trigger_timer(void *this_ptr);
-  virtual void set_update_interval (unsigned interval);
   virtual Bit8u get_actl_palette_idx(Bit8u index);
 
 private:
@@ -86,6 +85,7 @@ private:
 #endif // !BX_USE_CIRRUS_SMF
 
   static void   svga_timer_handler(void *);
+  static Bit64s svga_param_handler(bx_param_c *param, int set, Bit64s val);
   BX_CIRRUS_SMF void   svga_timer(void);
   BX_CIRRUS_SMF void   svga_modeupdate(void);
   BX_CIRRUS_SMF void   svga_update(void);
