@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: plugin.cc,v 1.8 2003/07/31 12:04:47 vruppert Exp $
+// $Id: plugin.cc,v 1.15 2006/01/21 09:28:49 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 // This file defines the plugin and plugin-device registration functions and
@@ -13,6 +13,7 @@
 /////////////////////////////////////////////////////////////////////////
 
 #include "bochs.h"
+#include "iodev/iodev.h"
 #include "plugin.h"
 
 #define LOG_THIS genlog->
@@ -27,13 +28,11 @@
 #define PLUGIN_FILENAME_FORMAT "bx_%s.dll"
 #endif
 
-
+extern "C" {
 
 void  (*pluginRegisterIRQ)(unsigned irq, const char* name) = 0;
 void  (*pluginUnregisterIRQ)(unsigned irq, const char* name) = 0;
 
-void  (* pluginResetSignal)(unsigned sig) = 0;
-
 void (*pluginSetHRQ)(unsigned val) = 0;
 void (*pluginSetHRQHackCallback)( void (*callback)(void) ) = 0;
 
@@ -41,14 +40,26 @@ int (*pluginRegisterIOReadHandler)(void *thisPtr, ioReadHandler_t callback,
                             unsigned base, const char *name, Bit8u mask) = 0;
 int (*pluginRegisterIOWriteHandler)(void *thisPtr, ioWriteHandler_t callback,
                              unsigned base, const char *name, Bit8u mask) = 0;
+int (*pluginUnregisterIOReadHandler)(void *thisPtr, ioReadHandler_t callback,
+                            unsigned base, Bit8u mask) = 0;
+int (*pluginUnregisterIOWriteHandler)(void *thisPtr, ioWriteHandler_t callback,
+                             unsigned base, Bit8u mask) = 0;
+int (*pluginRegisterIOReadHandlerRange)(void *thisPtr, ioReadHandler_t callback,
+                            unsigned base, unsigned end, const char *name, Bit8u mask) = 0;
+int (*pluginRegisterIOWriteHandlerRange)(void *thisPtr, ioWriteHandler_t callback,
+                             unsigned base, unsigned end, const char *name, Bit8u mask) = 0;
+int (*pluginUnregisterIOReadHandlerRange)(void *thisPtr, ioReadHandler_t callback,
+                            unsigned begin, unsigned end, Bit8u mask) = 0;
+int (*pluginUnregisterIOWriteHandlerRange)(void *thisPtr, ioWriteHandler_t callback,
+                             unsigned begin, unsigned end, Bit8u mask) = 0;
 int (*pluginRegisterDefaultIOReadHandler)(void *thisPtr, ioReadHandler_t callback,
                             const char *name, Bit8u mask) = 0;
 int (*pluginRegisterDefaultIOWriteHandler)(void *thisPtr, ioWriteHandler_t callback,
                              const char *name, Bit8u mask) = 0;
 int (*pluginRegisterTimer)(void *this_ptr, void (*funct)(void *),
-			Bit32u useconds, bx_bool continuous, 
-			bx_bool active, const char* name) = 0;
-void (*pluginActivateTimer)(unsigned id, Bit32u usec, bx_bool continuous) = 0;
+                            Bit32u useconds, bx_bool continuous, 
+bx_bool active, const char* name) = 0;
+                            void (*pluginActivateTimer)(unsigned id, Bit32u usec, bx_bool continuous) = 0;
 
 void (*pluginHRQHackCallback)(void);
 unsigned pluginHRQ = 0;
@@ -106,30 +117,92 @@ builtinSetHRQHackCallback( void (*callback)(void) )
 #endif
 }
 
-  static void
-builtinResetSignal( unsigned )
-{
-  pluginlog->panic("builtinResetSignal called, no plugin loaded?");
-}
-
   static int
 builtinRegisterIOReadHandler(void *thisPtr, ioReadHandler_t callback,
                             unsigned base, const char *name, Bit8u mask)
 {
+  int ret;
   BX_ASSERT (mask<8);
-  bx_devices.register_io_read_handler (thisPtr, callback, base, name, mask);
+  ret = bx_devices.register_io_read_handler (thisPtr, callback, base, name, mask);
   pluginlog->ldebug("plugin %s registered I/O read address at %04x", name, base);
-  return 0;
+  return ret;
 }
 
   static int
 builtinRegisterIOWriteHandler(void *thisPtr, ioWriteHandler_t callback,
                              unsigned base, const char *name, Bit8u mask)
 {
+  int ret;
   BX_ASSERT (mask<8);
-  bx_devices.register_io_write_handler (thisPtr, callback, base, name, mask);
+  ret = bx_devices.register_io_write_handler (thisPtr, callback, base, name, mask);
   pluginlog->ldebug("plugin %s registered I/O write address at %04x", name, base);
-  return 0;
+  return ret;
+}
+
+  static int
+builtinUnregisterIOReadHandler(void *thisPtr, ioReadHandler_t callback,
+                            unsigned base, Bit8u mask)
+{
+  int ret;
+  BX_ASSERT (mask<8);
+  ret = bx_devices.unregister_io_read_handler (thisPtr, callback, base, mask);
+  pluginlog->ldebug("plugin unregistered I/O read address at %04x", base);
+  return ret;
+}
+
+  static int
+builtinUnregisterIOWriteHandler(void *thisPtr, ioWriteHandler_t callback,
+                             unsigned base, Bit8u mask)
+{
+  int ret;
+  BX_ASSERT (mask<8);
+  ret = bx_devices.unregister_io_write_handler (thisPtr, callback, base, mask);
+  pluginlog->ldebug("plugin unregistered I/O write address at %04x", base);
+  return ret;
+}
+
+  static int
+builtinRegisterIOReadHandlerRange(void *thisPtr, ioReadHandler_t callback,
+                            unsigned base, unsigned end, const char *name, Bit8u mask)
+{
+  int ret;
+  BX_ASSERT (mask<8);
+  ret = bx_devices.register_io_read_handler_range (thisPtr, callback, base, end, name, mask);
+  pluginlog->ldebug("plugin %s registered I/O read addresses %04x to %04x", name, base, end);
+  return ret;
+}
+
+  static int
+builtinRegisterIOWriteHandlerRange(void *thisPtr, ioWriteHandler_t callback,
+                             unsigned base, unsigned end, const char *name, Bit8u mask)
+{
+  int ret;
+  BX_ASSERT (mask<8);
+  ret = bx_devices.register_io_write_handler_range (thisPtr, callback, base, end, name, mask);
+  pluginlog->ldebug("plugin %s registered I/O write addresses %04x to %04x", name, base, end);
+  return ret;
+}
+
+  static int
+builtinUnregisterIOReadHandlerRange(void *thisPtr, ioReadHandler_t callback,
+                            unsigned begin, unsigned end, Bit8u mask)
+{
+  int ret;
+  BX_ASSERT (mask<8);
+  ret = bx_devices.unregister_io_read_handler_range (thisPtr, callback, begin, end, mask);
+  pluginlog->ldebug("plugin unregistered I/O read addresses %04x to %04x", begin, end);
+  return ret;
+}
+
+  static int
+builtinUnregisterIOWriteHandlerRange(void *thisPtr, ioWriteHandler_t callback,
+                             unsigned begin, unsigned end, Bit8u mask)
+{
+  int ret;
+  BX_ASSERT (mask<8);
+  ret = bx_devices.unregister_io_write_handler_range (thisPtr, callback, begin, end, mask);
+  pluginlog->ldebug("plugin unregistered I/O write addresses %04x to %04x", begin, end);
+  return ret;
 }
 
   static int
@@ -154,8 +227,8 @@ builtinRegisterDefaultIOWriteHandler(void *thisPtr, ioWriteHandler_t callback,
 
   static int
 builtinRegisterTimer(void *this_ptr, void (*funct)(void *),
-			Bit32u useconds, bx_bool continuous, 
-			bx_bool active, const char* name)
+                        Bit32u useconds, bx_bool continuous, 
+                        bx_bool active, const char* name)
 {
   int id = bx_pc_system.register_timer (this_ptr, funct, useconds, continuous, active, name);
   pluginlog->ldebug("plugin %s registered timer %d", name, id);
@@ -297,10 +370,10 @@ plugin_load (char *name, char *args, plugintype_t type)
     plugin->name = name;
     plugin->args = args;
     plugin->initialized = 0;
-	
-	char plugin_filename[BX_PATHNAME_LEN], buf[BX_PATHNAME_LEN];
-	sprintf (buf, PLUGIN_FILENAME_FORMAT, name);
-	sprintf(plugin_filename, "%s%s", PLUGIN_PATH, buf);
+
+    char plugin_filename[BX_PATHNAME_LEN], buf[BX_PATHNAME_LEN];
+    sprintf (buf, PLUGIN_FILENAME_FORMAT, name);
+    sprintf(plugin_filename, "%s%s", PLUGIN_PATH, buf);
 
     // Set context so that any devices that the plugin registers will
     // be able to see which plugin created them.  The registration will
@@ -317,7 +390,7 @@ plugin_load (char *name, char *args, plugintype_t type)
       return;
     }
 
-	sprintf (buf, PLUGIN_INIT_FMT_STRING, name);
+    sprintf(buf, PLUGIN_INIT_FMT_STRING, name);
     plugin->plugin_init =  
       (int  (*)(struct _plugin_t *, enum plugintype_t, int, char *[])) /* monster typecast */
       lt_dlsym (plugin->handle, buf);
@@ -326,7 +399,7 @@ plugin_load (char *name, char *args, plugintype_t type)
         plugin_abort ();
     }
 
-	sprintf (buf, PLUGIN_FINI_FMT_STRING, name);
+    sprintf(buf, PLUGIN_FINI_FMT_STRING, name);
     plugin->plugin_fini = (void (*)(void)) lt_dlsym (plugin->handle, buf);
     if (plugin->plugin_init == NULL) {
         pluginlog->panic("could not find plugin_fini: %s", lt_dlerror ());
@@ -382,14 +455,21 @@ plugin_startup(void)
   pluginRegisterIRQ = builtinRegisterIRQ;
   pluginUnregisterIRQ = builtinUnregisterIRQ;
 
-  pluginResetSignal = builtinResetSignal;
-
   pluginSetHRQHackCallback = builtinSetHRQHackCallback;
   pluginSetHRQ = builtinSetHRQ;
   
   pluginRegisterIOReadHandler = builtinRegisterIOReadHandler;
   pluginRegisterIOWriteHandler = builtinRegisterIOWriteHandler;
+  
+  pluginUnregisterIOReadHandler = builtinUnregisterIOReadHandler;
+  pluginUnregisterIOWriteHandler = builtinUnregisterIOWriteHandler;
+  
+  pluginRegisterIOReadHandlerRange = builtinRegisterIOReadHandlerRange;
+  pluginRegisterIOWriteHandlerRange = builtinRegisterIOWriteHandlerRange;
 
+  pluginUnregisterIOReadHandlerRange = builtinUnregisterIOReadHandlerRange;
+  pluginUnregisterIOWriteHandlerRange = builtinUnregisterIOWriteHandlerRange;
+  
   pluginRegisterDefaultIOReadHandler = builtinRegisterDefaultIOReadHandler;
   pluginRegisterDefaultIOWriteHandler = builtinRegisterDefaultIOWriteHandler;
 
@@ -427,26 +507,20 @@ void pluginRegisterDeviceDevmodel(plugin_t *plugin, plugintype_t type, bx_devmod
     BX_ASSERT (devmodel != NULL);
     device->devmodel = devmodel;
     device->plugin = plugin;  // this can be NULL
-    device->use_devmodel_interface = 1;
-    device->device_init_mem = NULL;  // maybe should use 1 to detect any use?
-    device->device_init_dev = NULL;
-    device->device_reset = NULL;
-    device->device_load_state = NULL;
-    device->device_save_state = NULL;
     device->next = NULL;
 
     // Don't add every kind of device to the list.
     switch (type) {
       case PLUGTYPE_CORE:
-	// Core devices are present whether or not we are using plugins, so
-	// they are managed by the same code in iodev/devices.cc whether
-	// plugins are on or off.  
-	return; // Do not add core devices to the devices list.
+        // Core devices are present whether or not we are using plugins, so
+        // they are managed by the same code in iodev/devices.cc whether
+        // plugins are on or off.  
+        return; // Do not add core devices to the devices list.
       case PLUGTYPE_OPTIONAL:
       case PLUGTYPE_USER:
       default:
-	// The plugin system will manage optional and user devices only.
-	break;
+        // The plugin system will manage optional and user devices only.
+        break;
     }
 
     if (!devices)
@@ -507,28 +581,14 @@ void bx_init_plugins()
     // two loops
     for (device = devices; device; device = device->next)
     {
-      if (!device->use_devmodel_interface) {
-        if (device->device_init_mem != NULL) {
-            pluginlog->info("init_mem of '%s' plugin device by function pointer",device->name);
-            device->device_init_mem(BX_MEM(0));
-	}
-      } else {
-	pluginlog->info("init_mem of '%s' plugin device by virtual method",device->name);
-	device->devmodel->init_mem (BX_MEM(0));
-      }
+      pluginlog->info("init_mem of '%s' plugin device by virtual method",device->name);
+      device->devmodel->init_mem(BX_MEM(0));
     }
 
     for (device = devices; device; device = device->next)
     {
-      if (!device->use_devmodel_interface) {
-        if (device->device_init_dev != NULL) {
-            pluginlog->info("init_dev of '%s' plugin device by function pointer",device->name);
-            device->device_init_dev();
-	}
-      } else {
-	pluginlog->info("init_dev of '%s' plugin device by virtual method",device->name);
-	device->devmodel->init ();
-      }
+      pluginlog->info("init_dev of '%s' plugin device by virtual method",device->name);
+      device->devmodel->init();
     } 
 }
 
@@ -541,14 +601,9 @@ void bx_reset_plugins(unsigned signal)
     device_t *device;
     for (device = devices; device; device = device->next)
     {
-      if (!device->use_devmodel_interface) {
-        if (device->device_reset != NULL) {
-            pluginlog->info("reset of '%s' plugin device by function pointer",device->name);
-            device->device_reset(signal);
-        }
-      } else {
-	pluginlog->info("reset of '%s' plugin device by virtual method",device->name);
-	device->devmodel->reset (signal);
-      }
+      pluginlog->info("reset of '%s' plugin device by virtual method",device->name);
+      device->devmodel->reset(signal);
     }
 }
+
+}
